// $Id: Regression.h 865 2015-04-21 22:41:49Z ge $
// Copyright 2006 Gerald Weber
/// \file Regression.h
/// \brief contains the Regression function class
///
/// $Revision: 865 $
/// \author Gerald Weber <gweberbh@gmail.com>
#ifndef GBC_REGRESSION_H
#define GBC_REGRESSION_H "$Id: Regression.h 865 2015-04-21 22:41:49Z ge $"
#include <gsl/gsl_fit.h>
#include <valarray>
#include "MathAux.h"

namespace gbc
  {
  template<class _Tp=double>
  
  /// \brief Class which calculates linear regression
  ///
  /// \attention This class needs the GNU Scientific Library (GSL) 
  class Regression
    {
    public:
    typedef _Tp value_type;
    typedef std::pair<value_type,value_type> coefficient_type;

    value_type *x_data, *y_data;
    size_t x_data_size, y_data_size;
    value_type expoent;
    double c0, c1, cov00, cov01, cov11, chisq;

    Regression(value_type e=1.0): x_data_size(1), y_data_size(1), expoent(e)
      {
      x_data=new value_type[1];
      y_data=new value_type[1];
      }

    ~Regression(void)
      {
      delete[] x_data;
      delete[] y_data;
      }

    template<class _VectorTp>
    inline void x(const _VectorTp& xv)
      {
      if (x_data_size != xv.size()) 
        {
        x_data_size=xv.size();
        delete[] x_data;
        x_data=new value_type[xv.size()];
        }
      size_t n;
      for(n=0; n < xv.size(); ++n) x_data[n]=pow(xv[n],expoent);
      }

    template<class _VectorTp>
    inline void y(const _VectorTp& yv)
      {
      if (y_data_size != yv.size()) 
        {
        y_data_size=yv.size();
        delete[] y_data;
        y_data=new value_type[yv.size()];
        }
      size_t n;
      for(n=0; n < yv.size(); ++n) y_data[n]=yv[n];
      }

    template<class _Tp2, class _Tp3>
    inline void y(const std::map<_Tp2,_Tp3>& yv)
      {
      std::valarray<_Tp3> yvec(yv.size());
      convert(yv,yvec);
      y(yvec);
      }


    template<class _MapTp>
    /// \brief Assigns a map to the x and y vectors;
    inline void xy(const _MapTp& xy_map)
      {
      std::valarray<value_type> xv(xy_map.size()), yv (xy_map.size());
      convert(xy_map,xv,yv);
      x(xv); y(yv);
      }

    coefficient_type calculate_regression(void)
      {
      gsl_fit_linear (x_data, 1, y_data, 1, x_data_size, &c0, &c1, &cov00, &cov01, &cov11, &chisq);

      return coefficient_type(c0,c1);
      }

    template<class _VectorTp>
    coefficient_type calculate_regression(const _VectorTp& yv)
      {
      y(yv);
      return calculate_regression();
      }

    template<class _VectorTp>
    coefficient_type calculate_regression(const _VectorTp& xv, const _VectorTp& yv)
      {
      x(xv); y(yv);
      return calculate_regression();
      }

  };
}
#endif
